/*
 * A little and dumb security camera application with the LTI-Lib.
 * Revision list:
 * $Id: secucam.cpp,v 1.6 2003/04/26 11:05:39 wickel Exp $
 */

#include <ltiToUCam.h>

#ifdef _USE_PHILIPS_TOUCAM

#include <cstdio>
#include <ctime>
#include <unistd.h>
#include <iostream>

#include <ltiJPEGFunctor.h>
#include <ltiFastViewer.h>
#include <ltiDownsampling.h>
#include <ltiTemporalTemplate.h>
#include <ltiLispStreamHandler.h>
#include <ltiDraw.h>

using std::endl;
using std::cout;
using std::cerr;

void parseArgs(int argc, char*argv[], int& delay, bool& view, float& thresh) {
  delay=12;
  view=false;
  thresh=500.0f;
  for (int i=1; i<argc; i++) {
    if (*argv[i] == '-' || *argv[i] == '+') {
      //bool val=(*argv[i] == '+');
      switch (argv[i][1]) {
        // the minimum delay between taking two images
        case 's': delay=atoi(argv[++i]); break;
        // the threshold for considering two images different
        case 't': thresh=atof(argv[++i]); break;
        // enable the viewer
        case 'v': view=true; break;
        default: break;
      }
    } else {
      //
    }
  }
}

int main(int argc,char *argv[]) {

  int delay;
  bool view;
  float thresh;

  parseArgs(argc,argv,delay,view,thresh);

  lti::toUCam grabber;

  lti::image tmp;

  lti::toUCam::parameters p;
  // read camera parameter file
  std::ifstream gpf("tocam.dat");
  if (!gpf.is_open() || !gpf.good()) {
    // not found -> make one!
    std::cerr << "Parameter file not found, creating one!\n";
    std::ofstream gpf2("tocam.dat");
    lti::lispStreamHandler lsp(gpf2);
    p.write(lsp);
    gpf2.close();
  } else {
    lti::lispStreamHandler lsp(gpf);
    if (!p.read(lsp)) {
      std::cerr << "Parameter read failed\n";
      exit(1);
    }
    gpf.close();
  }

  // compute absolute threshold value
  // so the used can specify the value independent of the image
  // size
  thresh=p.size.x*p.size.y/thresh;

  grabber.setParameters(p);
  lti::saveJPEG saver;

  char buf[32];
  char buf2[32];

  lti::draw<lti::rgbPixel> drawer;
  lti::temporalTemplate mhi;
  lti::downsampling sampler;
  lti::channel result;
  bool init=true;
  lti::image tmp2;

  struct tm timestamp,timestamp2;
  int frame=0;
  time_t ttime;
  lti::fastViewer viewer;

  // main loop
  while (grabber.apply(tmp)) {
    time(&ttime);
    struct tm* t=localtime_r(&ttime,&timestamp);
    strcpy(buf2,asctime(t));
    buf2[strlen(buf2)-1]='\0';

    sampler.apply(tmp,tmp2);
    mhi.apply(tmp2,result);

    float s=result.sumOfElements();
    if (s > thresh || init) {

      // draw time stamp into image
      drawer.use(tmp);
      drawer.setColor(lti::Black);
      drawer.box(0,0,192,8,true);
      drawer.setColor(lti::White);
      drawer.text(buf2);

      init=false;
      // compare the time stamps
      if (memcmp(&timestamp,&timestamp2,sizeof(timestamp)) == 0) {
        // equal -> use frame number for image file name
        sprintf(buf,"/tmp/tou/%04d%02d%02d-%02d%02d%02d_%02d.jpg",
              t->tm_year+1900,t->tm_mon+1,t->tm_mday,t->tm_hour,t->tm_min,t->tm_sec,++frame);
      } else {
        // different -> no need for frame no.
        sprintf(buf,"/tmp/tou/%04d%02d%02d-%02d%02d%02d.jpg",
              t->tm_year+1900,t->tm_mon+1,t->tm_mday,t->tm_hour,t->tm_min,t->tm_sec);
        memcpy(&timestamp2,&timestamp,sizeof(timestamp));
        frame=0;
      }
      // save image
      saver.save(buf,tmp);
    } else {
      // no movement->make image black so viewer does not show anything
      tmp.fill(lti::Black);
    }
    if (delay > 0) {
      sleep(delay);
    }
    if (view) {
      viewer.show(tmp);
    }
  }

}

#else

#include <iostream>

using std::endl;
using std::cout;
using std::cerr;

// no toUCam camera found
int main(int argc,char *argv[]) {
  std::cout<<"Sorry, but no toUCam support found with your LTI-Lib"<<endl;
  std::cout<<"You may want to recompile it enabling the camera support in "
           << "ltilib/src/io/ltiHardware.h" << endl;
  return 1;
}
#endif

