/*
 * Copyright (C) 1998, 1999, 2000, 2001
 * Lehrstuhl fuer Technische Informatik, RWTH-Aachen, Germany
 *
 * This file is part of the LTI-Computer Vision Library (LTI-Lib)
 *
 * The LTI-Lib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License (LGPL)
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * The LTI-Lib is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with the LTI-Lib; see the file LICENSE.  If
 * not, write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 */


/*----------------------------------------------------------------
 * project ....: LTI Digital Image/Signal Processing Library
 * file .......: ltiMergergIToImage.cpp
 * authors ....: Pablo Alvarado, Stefan Syberichs, Thomas Rusert
 * organization: LTI, RWTH Aachen
 * creation ...: 19.04.99
 * revisions ..: $Id: ltiMergergIToImage.cpp,v 1.1 2003/11/14 16:07:22 alvarado Exp $
 */

#include "ltiMergergIToImage.h"

namespace lti {

  // merge rgI channels to image

  // returns the name of this type
  const char* mergergIToImage::getTypeName() const {
    return "mergergIToImage";
  }

  // returns a pointer to a clone of the functor.
  functor* mergergIToImage::clone() const {
    return (new mergergIToImage(*this));
  }

  bool mergergIToImage::apply(const matrix<float>& c1,
                              const matrix<float>& c2,
                              const matrix<float>& c3,
                              image& img) const {
    int xSize,ySize;
    int i;
    point p;            // coordinates
    rgbPixel pix;       // pixel structure

    xSize=c1.columns();
    ySize=c1.rows();
    if((c2.columns()!=xSize)||
       (c2.rows()!=ySize)||
       (c3.columns()!=xSize)||
       (c3.rows()!=ySize)) {
      setStatusString("sizes of channels do not match");
      return false;
    }

    img.resize(ySize,xSize,rgbPixel(),false,false);

    for (p.y=0;p.y<ySize;p.y++)
      for (p.x=0;p.x<xSize;p.x++) {
	i=int(c3.at(p)*765.0f); // 3*255 = 765
	pix.setRed(ubyte(c1.at(p)*i));
	pix.setGreen(ubyte(c2.at(p)*i));
	pix.setBlue(ubyte(i-pix.getRed()-pix.getGreen()));
	img.at(p) = pix;
      }

    return true;
  }

  bool mergergIToImage::apply(const channel8& c1,
                              const channel8& c2,
                              const channel8& c3,
                              image& img) const {

    int xSize,ySize;
    int i;
    point p;          // coordinates
    rgbPixel pix;     // pixel structure

    // check the size of the three channels
    xSize=c1.columns();
    ySize=c1.rows();
    if ((c2.columns()!=xSize)||
        (c2.rows()!=ySize)||
        (c3.columns()!=xSize)||
        (c3.rows()!=ySize)) {
      setStatusString("sizes of channels do not match");
      return false; // if sizes don't match
    }

    // new merged image
    img.resize(ySize,xSize,rgbPixel(),false,false);

    for (p.y=0;p.y<ySize;p.y++)
      for (p.x=0;p.x<xSize;p.x++) {
        i=c3.at(p)*3;
        pix.setRed(ubyte(int(c1.at(p)*i)/255));
        pix.setGreen(ubyte(int(c2.at(p)*i)/255));
        pix.setBlue(ubyte(i-pix.getRed()-pix.getGreen()));
        img.at(p) = pix;
      }

    return true;
  }

  bool mergergIToImage::apply(const float& c1,
                              const float& c2,
                              const float& c3,
                              rgbPixel& pixel) const {

    int i=int(c3*765.0f); // 3*255 = 765
    pixel.setRed(ubyte(c1*i));
    pixel.setGreen(ubyte(c2*i));
    pixel.setBlue(ubyte(i-pixel.getRed()-pixel.getGreen()));

    return true;
  }

  bool mergergIToImage::apply(const ubyte& c1,
                              const ubyte& c2,
                              const ubyte& c3,
                              rgbPixel& pixel) const {

    int i=c3*3;
    pixel.setRed(ubyte(int(c1*i)/255));
    pixel.setGreen(ubyte(int(c2*i)/255));
    pixel.setBlue(ubyte(i-pixel.getRed()-pixel.getGreen()));

    return true;
  }


} // end of namespace

