/*
 * Copyright (C) 1998, 1999, 2000, 2001
 * Lehrstuhl fuer Technische Informatik, RWTH-Aachen, Germany
 *
 * This file is part of the LTI-Computer Vision Library (LTI-Lib)
 *
 * The LTI-Lib is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License (LGPL)
 * as published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * The LTI-Lib is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with the LTI-Lib; see the file LICENSE.  If
 * not, write to the Free Software Foundation, Inc., 59 Temple Place -
 * Suite 330, Boston, MA 02111-1307, USA.
 */


/* -----------------------------------------------------------------------
 * project ....: LTI Digitale Bild/Signal Verarbeitungsbibliothek
 * file .......: ltiConfigFile.cpp
 * authors ....: Ruediger Weiler
 * organization: LTI, RWTH Aachen
 * creation ...: 23.10.2001
 * revisions ..: $Id: ltiConfigFileHandler.cpp,v 1.26 2003/10/30 10:03:01 alvarado Exp $
 */

#include "ltiConfigFileHandler.h"
#include <iostream>
#include <sstream>
#include <fstream>
#include <cstdio>
#include <cstdlib>

namespace lti {
  // --------------------------------------------------
  // configFileHandler::parameters
  // --------------------------------------------------

  // default constructor
  configFileHandler::parameters::parameters()
    : ioFunctor::parameters() {
    //TODO: Initialize your parameter values!
    // If you add more parameters manually, do not forget to do following:
    // 1. indicate in the default constructor the default values
    // 2. make sure that the copy member also copy your new parameters
    // 3. make sure that the read and write members also read and
    //    write your parameters
    filename = "";
    formatOutput = true;;
  }

  // copy constructor
  configFileHandler::parameters::parameters(const parameters& other)
    : ioFunctor::parameters()  {
    copy(other);
  }

  // destructor
  configFileHandler::parameters::~parameters() {
  }

  // get type name
  const char* configFileHandler::parameters::getTypeName() const {
    return "configFileHandler::parameters";
  }

  // copy member

  configFileHandler::parameters&
    configFileHandler::parameters::copy(const parameters& other) {
# ifndef _LTI_MSC_VER
    // MS Visual C++ 6 is not able to compile this...
    ioFunctor::parameters::copy(other);
# else
    // ...so we have to use this workaround.
    // Conditional on that, copy may not be virtual.
    ioFunctor::parameters& (ioFunctor::parameters::* p_copy)
      (const ioFunctor::parameters&) =
      ioFunctor::parameters::copy;
    (this->*p_copy)(other);
# endif

    formatOutput = other.formatOutput;

    return *this;
  }

  // alias for copy member
  configFileHandler::parameters&
    configFileHandler::parameters::operator=(const parameters& other) {
    return copy(other);
  }

  // clone member
  functor::parameters* configFileHandler::parameters::clone() const {
    return new parameters(*this);
  }

  /*
   * write the parameters in the given ioHandler
   * @param handler the ioHandler to be used
   * @param complete if true (the default) the enclosing begin/end will
   *        be also written, otherwise only the data block will be written.
   * @return true if write was succeful
   */
# ifndef _LTI_MSC_VER
  bool configFileHandler::parameters::write(ioHandler& handler,
                                         const bool& complete) const
# else
  bool configFileHandler::parameters::writeMS(ioHandler& handler,
                                           const bool& complete) const
# endif
  {
    bool b = true;
    if (complete) {
      b = handler.writeBegin();
    }

    if (b) {
      lti::write(handler,"formatOutput",formatOutput);
    }

# ifndef _LTI_MSC_VER
    // This is the standard C++ code, which MS Visual C++ 6 is not able to
    // compile...
    b = b && ioFunctor::parameters::write(handler,false);
# else
    bool (ioFunctor::parameters::* p_writeMS)(ioHandler&,const bool&) const =
      ioFunctor::parameters::writeMS;
    b = b && (this->*p_writeMS)(handler,false);
# endif

    if (complete) {
      b = b && handler.writeEnd();
    }

    return b;
  }

# ifdef _LTI_MSC_VER
  bool configFileHandler::parameters::write(ioHandler& handler,
                                         const bool& complete) const {
    // ...we need this workaround to cope with another really awful MSVC bug.
    return writeMS(handler,complete);
  }
# endif

  /*
   * read the parameters from the given ioHandler
   * @param handler the ioHandler to be used
   * @param complete if true (the default) the enclosing begin/end will
   *        be also read, otherwise only the data block will be read.
   * @return true if write was succeful
   */
# ifndef _LTI_MSC_VER
  bool configFileHandler::parameters::read(ioHandler& handler,
                                        const bool& complete)
# else
  bool configFileHandler::parameters::readMS(ioHandler& handler,
                                          const bool& complete)
# endif
  {
    bool b = true;
    if (complete) {
      b = handler.readBegin();
    }

    if (b) {
      lti::read(handler,"formatOutput",formatOutput);
    }

# ifndef _LTI_MSC_VER
    // This is the standard C++ code, which MS Visual C++ 6 is not able to
    // compile...
    b = b && ioFunctor::parameters::read(handler,false);
# else
    bool (ioFunctor::parameters::* p_readMS)(ioHandler&,const bool&) =
      ioFunctor::parameters::readMS;
    b = b && (this->*p_readMS)(handler,false);
# endif

    if (complete) {
      b = b && handler.readEnd();
    }

    return b;
  }

# ifdef _LTI_MSC_VER
  bool configFileHandler::parameters::read(ioHandler& handler,
                                        const bool& complete) {
    // ...we need this workaround to cope with another really awful MSVC
    // bug.
    return readMS(handler,complete);
  }
# endif

  // --------------------------------------------------
  // configFileHandler
  // --------------------------------------------------

  inline bool isLineStartComment(unsigned char c) {
    //check ih c is a comment
    return ((c=='*') || (c=='#') || (c==';') || (c=='/'));
  }

  inline bool isLineMidComment(unsigned char c) {
    //is c a comment in the line
    return (c==';');
  }

  inline bool isWhiteSpace(unsigned char c) {
    //is c a tab a space or a whitespace
    return ((c==' ')||(c=='\t')||(c==160)); //160 is whitespace
  }

  inline bool isLegalChar(unsigned char c) {
    return ( c >= 32 );
  }

  inline void killSpaces(std::string &str) {
    unsigned int start =0;
    unsigned int end =0;
    for(start=0; start < static_cast<unsigned int>(str.size()); start++)
      if(!isWhiteSpace(str[start])) {
        break;
      }
    for(end=str.size(); end>0; end--) {
      if( !( (str[end-1]==' ') || (str[end-1]=='\t') || (str[end-1]==char(0xa0)) ) ) {
        break;
      }
    }
    str = str.substr(start,end-start);
  }

  inline void lowerString(std::string &str) {
    //transform a string to lower case
    for(unsigned int x=0; x < static_cast<unsigned int>(str.size()); x++)
      str[x] = tolower(str[x]);
  }

  // default constructor
  configFileHandler::configFileHandler()
    : ioFunctor(){

    // comment the attributes of your functor
    // If you add more attributes manually, do not forget to do following:
    // 1. indicate in the default constructor the default values
    // 2. make sure that the copy member also copy your new attributes, or
    //    to ensure there, that these attributes are properly initialized.
    dictionary.clear();

    // create an instance of the parameters with the default values
    parameters defaultParameters;
    // set the default parameters
    setParameters(defaultParameters);
  }

  // copy constructor
  configFileHandler::configFileHandler(const configFileHandler& other)
    : ioFunctor()  {
    copy(other);
  }

  // destructor
  configFileHandler::~configFileHandler() {
    dictionary.clear();
  }

  // returns the name of this type
  const char* configFileHandler::getTypeName() const {
    return "configFileHandler";
  }

  // which apply is faster
  functor::eFasterApply configFileHandler::theFaster() const {
    return OnCopy;
  }

  // copy member
  configFileHandler&
    configFileHandler::copy(const configFileHandler& other) {
      ioFunctor::copy(other);

    //TODO: comment the attributes of your functor
    // If you add more attributes manually, do not forget to do following:
    // 1. indicate in the default constructor the default values
    // 2. make sure that the copy member also copy your new attributes, or
    //    to ensure there, that these attributes are properly initialized.
    dictionary = other.dictionary;

    return (*this);
  }

  // alias for copy member
  configFileHandler&
    configFileHandler::operator=(const configFileHandler& other) {
    return (copy(other));
  }

  // clone member
  functor* configFileHandler::clone() const {
    return new configFileHandler(*this);
  }

  // return parameters
  const configFileHandler::parameters&
    configFileHandler::getParameters() const {
    const parameters* par =
      dynamic_cast<const parameters*>(&functor::getParameters());
    if(isNull(par)) {
      throw invalidParametersException(getTypeName());
    }
    return *par;
  }

  // -------------------------------------------------------------------
  // The apply-methods!
  // -------------------------------------------------------------------

  bool configFileHandler::apply(){
     // Get parameters
    const parameters& param = getParameters();
    return loadFile(param.filename);
  }


  bool configFileHandler::loadFile(const std::string &aFileName){

    std::ifstream file(aFileName.c_str());
    std::string input;
    int lineNr = 0;
    configEntry newEntry;
    std::string actSection = "";
    std::string prolog = "";
    std::string section = "";
    std::string comment = "";
    std::string key = "";
    std::string value = "";
    std::string errorS = "";

    // check if file is open
    if(!file.is_open()){
      std::string error = "Unable to open file: " + aFileName;
      setStatusString(error.c_str());
      return false;
    }

    dictionary.clear(); //clears the internal list

    while(std::getline(file,input)){ //go line by line through the file

      errorS = splitLine(input,prolog,section,key,value,comment);
      if( errorS != "") {
         //generatin the ostream is the securest way for converting
        std::ostringstream os;
        os << lineNr;
        std::string error = "Line  ";
        error = std::string(os.str()) + " : " + errorS;
        setStatusString(error.c_str());
        return false;
      }

      // generate an entry in the dictionary
      if( section=="" ){ // is normal entry within section
        if( (key!="")||(comment!="") ){
          newEntry.prolog = prolog;
          newEntry.section = actSection;
          newEntry.key = key;
          newEntry.value = value;
          newEntry.comment = comment;
          newEntry.sectionDef = false;

          dictionary.push_back(newEntry);
        }
      }
      else { // is section definition
        actSection = section;
        newEntry.sectionDef = true;
        newEntry.prolog = "";
        newEntry.section = actSection;
        newEntry.key = "";
        newEntry.value = "";
        newEntry.comment = comment;

        dictionary.push_back(newEntry);
      }

      lineNr++;
    }
    return true;
  }

  bool configFileHandler::loadFile() {
    return apply();
  }

  bool configFileHandler::saveFile(const std::string &aFileName) const {

    std::ofstream file(aFileName.c_str());
    std::string temp="";

    std::list<configEntry>::const_iterator itc = dictionary.begin();

    //check if the file is open
    if( !file.is_open() ){
      std::string error = "Unable to open file: " + aFileName;
      setStatusString(error.c_str());
      return false;
    }

    std::map<std::string,int> keyLenMap;
    if (getParameters().formatOutput) {
      getMaxKeyLengths(keyLenMap);
    }
    int keyLen;

    while(itc != dictionary.end()){
      //run through the list
      if (getParameters().formatOutput && (keyLenMap.find((*itc).section)!=keyLenMap.end()) ) {
        keyLen = keyLenMap[(*itc).section];
      }
      else {
        keyLen = 0;
      }

      composeLine((*itc).prolog, (*itc).section, (*itc).key, (*itc).value,
                  (*itc).comment, (*itc).sectionDef, keyLen, temp);

      if( temp != "") {
        file << temp << std::endl;
      }
      itc++;
    }

    file.close();
    return true;
  }

  bool configFileHandler::saveFile() const {
    return saveFile(getParameters().filename);
  }


  // setValue members ---------------------------------------------------------

  bool configFileHandler::getValue(const std::string &section,
                                   const std::string &key,
                                   std::string &value,
                                   std::string &comment) const {
    // get a string at the point section and key
    // value = ""; leave unaltered
    comment = "";
    //allways lower case section and key names
    std::string sec = section; killSpaces(sec);
    lowerString(sec); // for case insensitive search
    std::string k = key; killSpaces(k);
    lowerString(k); // for case insensitive search

    std::list<configEntry>::const_iterator itc = dictionary.begin();

    while(itc != dictionary.end()){
      std::string thisSec = (*itc).section;
      lowerString(thisSec);
      std::string thisKey = (*itc).key;
      lowerString(thisKey);
      //run through the list and compare case insensitive
      if(thisSec == sec &&
         thisKey == k) {

        value = (*itc).value;
        comment = (*itc).comment;
        return true;
      }
      itc++;
    }
    return false;
  }

  bool configFileHandler::getValue(const std::string &section,
                                   const std::string &key,
                                   bool &value,
                                   std::string &comment) const {
    //get the string at the desired point and convert it to a boolean
    std::string erg;
    if( getValue(section,key,erg,comment) ){
      //convert the string erg to a boolean
      if(erg[0]=='y'|| erg[0]=='Y'|| erg[0]=='1'|| erg[0]=='t'|| erg[0]=='T') {
        value = true;
        return true;
      }
      else if(erg[0]=='n'|| erg[0]=='N'|| erg[0]=='0'|| erg[0]=='f'|| erg[0]=='F') {
        value = false;
        return true;
      }
      //if the string is not recognized return false
    }
    //value = false; //if nothing found leave unaltered
    return false; //and return false
  }

  bool configFileHandler::getValue(const std::string &section,
                                   const std::string &key,
                                   double &value,
                                   std::string &comment) const {
    //get the string at the desired point and convert it to a double
    std::string erg;
    if( getValue(section,key,erg,comment) ){
      //convert the string erg to double
      value = atof(erg.c_str());
      return true;
    }
    //value = 0; //if nothing found leave unaltered
    return false; //and return false
  }

  bool configFileHandler::getValue(const std::string &section,
                                   const std::string &key,
                                   float &value,
                                   std::string &comment) const {
    //get the string at the desired point and convert it to a double
    std::string erg;
    if( getValue(section,key,erg,comment) ){
      //convert the string erg to double
      value = static_cast<float>(atof(erg.c_str()));
      return true;
    }
    //value = 0; //if nothing found leave unaltered
    return false; //and return false
  }

  bool configFileHandler::getValue(const std::string &section,
                                   const std::string &key,
                                   int &value,
                                   std::string &comment) const {
    //get the string at the desired point and convert it to an int
    std::string erg;
    if( getValue(section,key,erg,comment) ){
      //convert the string erg to int
      value = atoi(erg.c_str());
      return true;
    }
    //value = 0; //if nothing found leave unaltered
    return false; //and return false
  }

  bool configFileHandler::getValue(const std::string &section,
                                   const std::string &key,
                                   unsigned int &value,
                                   std::string &comment) const {
    //get the string at the desired point and convert it to an int
    std::string erg;
    if( getValue(section,key,erg,comment) ){
      //convert the string erg to int
      value = static_cast<unsigned int>(atoi(erg.c_str()));
      return true;
    }
    //value = 0; //if nothing found leave unaltered
    return false; //and return false
  }



  // setValue members ---------------------------------------------------------

  bool configFileHandler::setValue(const std::string &section,
                                   const std::string &key,
                                   const std::string &value,
                                   const std::string &comment) {

    std::list<configEntry>::iterator it,itSec;
    configEntry newEntry;

    std::string sec = section;
    std::string k = key;
    std::string v = value;
    std::string com = comment;

    std::string thisSec;  // temporary help variables
    std::string thisKey;

    killSpaces(sec);
    killSpaces(k);
    lowerString(sec); // for case insensitive search
    lowerString(k);   // for case insensitive search

    killSpaces(v);
    killSpaces(com);

    // no empty lines!
    if( (sec=="")&&(k=="")&&(v=="")&&(com=="") ) {
      setStatusString("Error: no empty lines are allowed.");
      return false;
    }

    // no values without keys!
    if( (k=="") && (v!="") ) {
      setStatusString("Error: no values without keys.");
      return false;
    }


    if( k=="" ) { // insert a new section or a comment or a comment to a section
      newEntry.section = section;
      newEntry.sectionDef = false;
      newEntry.key = "";
      newEntry.value = "";
      //search the section
      bool sectionExist = false;
      it = dictionary.end();
      while(it != dictionary.begin()){
        it--;

        thisSec = (*it).section;
        lowerString(thisSec);
        thisKey = (*it).key;
        lowerString(thisKey);

        if(thisSec == sec && thisKey == ""){
          sectionExist = true;
          break;
        }
      }
      if( sectionExist ){ //there is a section found
        // insert new comment after the position it
        it++; //the insert funktion insert always befor the iterator
        newEntry.comment = "# " + com;
        dictionary.insert(it,newEntry);
        return true;
      }
      else { //no section found
        if( sec != "" ) {//insert a new section
          newEntry.sectionDef = true;
          newEntry.comment = com;
          dictionary.push_back(newEntry);
        }
        else {  // global comment
          newEntry.sectionDef = false;
          newEntry.comment = "# " + com;
          dictionary.push_back(newEntry);
        }
        return true;
      }
    }
    else { //insert or update a key value pair
      newEntry.section = section;
      //search the section and/or the key
      bool sectionExist = false;
      bool keyExist = false;
      it = dictionary.end();
      while(it != dictionary.begin()){
        it--;

        thisSec = (*it).section;
        lowerString(thisSec);
        thisKey = (*it).key;
        lowerString(thisKey);

        if(thisSec == sec && !sectionExist){
          sectionExist = true;
          itSec = it;
        }
        if(thisSec == sec && thisKey == k){
          keyExist = true;
          break;
        }
      }

      if( keyExist ) {//the key exist in the section
        //update entry
        (*it).value = v;
        (*it).comment = com;
        (*it).prolog = "";
        return true;
      }
      else if( sectionExist ) {//the section exist
        // insert new key and value after the last entry in the section
        itSec++; //the insert funktion insert always befor the iterator
        newEntry.prolog = "";
        newEntry.key = key;
        newEntry.sectionDef = false;
        newEntry.value = v;
        newEntry.comment = com;
        dictionary.insert(itSec,newEntry);
        return true;
      }
      else {//no section and no key
        //insert a new section
        newEntry.prolog = "";
        newEntry.sectionDef = true;
        newEntry.key = "";
        newEntry.value = "";
        newEntry.comment = "";
        dictionary.push_back(newEntry);
        //insert key and value
        newEntry.sectionDef = false;
        newEntry.prolog = "";
        newEntry.key = key;
        newEntry.value = v;
        newEntry.comment = com;
        dictionary.push_back(newEntry);
        return true;
      }
    }
    return false;
  }


  bool configFileHandler::setValue(const std::string &section,
                                   const std::string &key,
                                   const bool &value,
                                   const std::string &comment) {
    std::string wert="false";
    if(value == true) wert = "true";
    return setValue(section,key,wert,comment);
  }

  bool configFileHandler::setValue(const std::string &section,
                                   const std::string &key,
                                   const double &value,
                                   const std::string &comment) {
    //generating the ostream is the securest way for converting
    std::ostringstream os;
    os << value;
    return setValue(section,key,os.str(),comment);
  }

  bool configFileHandler::setValue(const std::string &section,
                                   const std::string &key,
                                   const float &value,
                                   const std::string &comment) {
    //generating the ostream is the securest way for converting
    std::ostringstream os;
    os << value;
    return setValue(section,key,os.str(),comment);
  }

  bool configFileHandler::setValue(const std::string &section,
                                   const std::string &key,
                                   const int &value,
                                   const std::string &comment) {
    //generating the ostream is the securest way for converting
    std::ostringstream os;
    os << value;
    return setValue(section,key,os.str(),comment);
  }

  bool configFileHandler::setValue(const std::string &section,
                                   const std::string &key,
                                   const unsigned int &value,
                                   const std::string &comment) {

    //generating the ostream is the securest way for converting
    std::ostringstream os;
    os << value;
    return setValue(section,key,os.str(),comment);
  }



  // manipulation -------------------------------------------------------------

  void configFileHandler::sortSections() {
    dictionary.sort();
  }

  bool configFileHandler::deleteSection(const std::string &section) {
    std::list<configEntry>::iterator it;
    std::string sec = section; killSpaces(sec);
    bool del= false;

    it=dictionary.begin();
    while(it!=dictionary.end()){
      if( (*it).section == sec ){
        it = dictionary.erase(it);
        del = true;
      }
      else
        it++;
    }
    return del;
  }

  bool configFileHandler::deleteKey(const std::string &section,
                                    const std::string &key) {
    std::list<configEntry>::iterator it;
    std::string sec = section; killSpaces(sec);
    std::string ky = key; killSpaces(ky);
    bool del= false;

    it=dictionary.begin();
    while(it != dictionary.end()){
      if( (*it).section == sec && (*it).key == ky ){
        if( (ky == "" && (*it).comment != "") || (ky != "") ){
          it = dictionary.erase(it);
          del = true;
        }
        else it++;
      }
      else it++;
    }
    return del;
  }

  void configFileHandler::deleteConfigFile() {
    dictionary.clear();
  }


  // other --------------------------------------------------------------------

  std::string configFileHandler::splitLine(const std::string& inputLine,
                                           std::string& prolog,
                                           std::string& section,
                                           std::string& key,
                                           std::string& value,
                                           std::string& comment) const {

    enum { PROLOG, SECTION, SECTIONEPILOG, KEY, VALUE, COMMENT } state;

    // initialize empty
    prolog = section = key = value = comment = "";

    //initial status
    state = PROLOG;

    // scan characters
    for(unsigned int x=0; x<inputLine.size(); x++){

      unsigned char c = inputLine[x];

      switch(state) {
      case PROLOG:
        if( isLineStartComment(c) ){
          prolog = "";
          comment = inputLine;
          x=inputLine.size();
        }
        else if( c == '[' )
          state = SECTION;
        else if( isWhiteSpace(c) )
          prolog += c;
        else if( c == '=' )
          return "no key name specified";
        else if( c== ']' )
          return "the '[' and a name is missing";
        else if( !isLegalChar(c) )
          return "illegal character found";
        else {
          state = KEY;
          key += c;
        }
        break;
      case SECTION:
        if( c == ']' )
          state = SECTIONEPILOG;
        else if( !isLegalChar(c) )
          return "illegeal character found";
        else if( isLineStartComment(c) )
          return "comment character in a section definition";
        else if( c== '[' )
          return "expect the ']' not the '[' character";
        else
          section += c;
        break;
      case SECTIONEPILOG:
        if( isWhiteSpace(c) )
          break; //do nothing
        else if( isLineMidComment(c) )
          state = COMMENT;
        else
          return "only comments are allowed after a section";
        break;
      case KEY:
        if( c == '=' )
          state = VALUE;
        else if( isLegalChar(c) )
          key += c;
        else
          return "wrong character in key expression";
        break;
      case VALUE:
        if( isLineMidComment(c) )
          state = COMMENT;
        else if( isLegalChar(c)||isWhiteSpace(c) )
          value += c;
        else
          return "wrong character in value expression";
        break;
      case COMMENT:
        comment += c;
        break;
      default: //default is a comment line
        prolog = "";
        comment = inputLine;
        x=inputLine.size();
        break;
      }
    }

    if( state == KEY ) //no = is found in this line
      return "the '=' is missing";

    killSpaces(section);
    //lowerString(section);
    killSpaces(key);
    //lowerString(key);
    killSpaces(value);

    return "";
  }


  void configFileHandler::composeLine(const std::string& prolog,
                                      const std::string& section,
                                      const std::string& key,
                                      const std::string& value,
                                      const std::string& comment,
                                      bool secDef,
                                      int keyLength,
                                      std::string& outputLine) const {

    outputLine = "";

    if(secDef) { //a section definition

      if (getParameters().formatOutput) {
        outputLine = "\n";
      }

      if( (section!="")&&(comment=="") ) {
        outputLine = outputLine + "[" + section +"]";
      }
      else if( (section!="")&&(comment!="") ) {
        outputLine = outputLine + "[" + section +"] ;" + comment;
      }
      else {
        outputLine = "";
      }

    }
    else { // a normal comment or key-value line

      std::string fillStr="";
      if (keyLength > static_cast<int>(key.size())) {
        fillStr = std::string(keyLength-key.size(),' ');
      }

      if( (key=="") && (value=="") && (comment!="") ) {
        outputLine = comment;
      }
      else if( (key!="")&&(comment=="") ) {
        outputLine = prolog + key + fillStr + " = " + value;
      }
      else if( (key!="")&&(comment!="") ) {
        outputLine = prolog + key + fillStr + " = " + value + " ;" + comment;
      }
      else {
        outputLine = "";
      }
    }

  }

  int configFileHandler::getMaxKeyLengths(std::map<std::string,int>& maxKeyLengths) const {

    std::list<configEntry>::const_iterator it;
    maxKeyLengths.clear();

    int maxLength = 0;
    int thisLength;
    it = dictionary.begin();
    while (it != dictionary.end()) {

      if ((*it).section!="") {
        thisLength = (*it).key.size();
        // if first key found for a section
        if (!(maxKeyLengths.find((*it).section)!=maxKeyLengths.end())) {
          maxKeyLengths[(*it).section] = thisLength;
        }
        else {
          // if current size larger than last stored, replace it
          if (maxKeyLengths[(*it).section] < thisLength) {
            maxKeyLengths[(*it).section] = thisLength;
          }
        }

        // determine maxLength
        if (maxLength < thisLength) {
          maxLength = thisLength;
        }

      }
      it++;
    }

    return maxLength;
  }


}




